@@ -32,6 +32,7 @@
 #include	"md_u.h"
 #include	<sys/wait.h>
 #include	<sys/signal.h>
+#include	<values.h>
 
 static void alert(char *event, char *dev, char *disc, char *mailaddr, char *cmd);
 
@@ -45,8 +46,8 @@ static char *percentalerts[] = {
 
 int Monitor(mddev_dev_t devlist,
 	    char *mailaddr, char *alert_cmd,
-	    int period, int daemonise, int scan,
-	    char *config)
+	    int period, int daemonise, int scan, int oneshot,
+	    char *config, int test)
 {
 	/*
 	 * Every few seconds, scan every md device looking for changes
@@ -97,6 +98,7 @@ int Monitor(mddev_dev_t devlist,
 		int err;
 		char *spare_group;
 		int active, working, failed, spare, raid;
+		int expected_spares;
 		int devstate[MD_SB_DISKS];
 		int devid[MD_SB_DISKS];
 		int percent;
@@ -149,8 +151,9 @@ int Monitor(mddev_dev_t devlist,
 			st->utime = 0;
 			st->next = statelist;
 			st->err = 0;
-			st->devnum = -1;
+			st->devnum = MAXINT;
 			st->percent = -2;
+			st->expected_spares = mdlist->spare_disks;
 			if (mdlist->spare_group)
 				st->spare_group = strdup(mdlist->spare_group);
 			else
@@ -167,8 +170,9 @@ int Monitor(mddev_dev_t devlist,
 			st->utime = 0;
 			st->next = statelist;
 			st->err = 0;
-			st->devnum = -1;
+			st->devnum = MAXINT;
 			st->percent = -2;
+			st->expected_spares = -1;
 			st->spare_group = NULL;
 			statelist = st;
 		}
@@ -176,19 +180,22 @@ int Monitor(mddev_dev_t devlist,
 
 
 	while (! finished) {
+		int new_found = 0;
 		struct state *st;
 
 		if (mdstat)
 			free_mdstat(mdstat);
-		mdstat = mdstat_read();
+		mdstat = mdstat_read(oneshot?0:1);
 
 		for (st=statelist; st; st=st->next) {
 			mdu_array_info_t array;
-			struct mdstat_ent *mse;
+			struct mdstat_ent *mse = NULL, *mse2;
 			char *dev = st->devname;
 			int fd;
-			int i;
+			unsigned int i;
 
+			if (test)
+				alert("TestMessage", dev, NULL, mailaddr, alert_cmd);
 			fd = open(dev, O_RDONLY);
 			if (fd < 0) {
 				if (!st->err)
@@ -217,17 +224,21 @@ int Monitor(mddev_dev_t devlist,
 				close(fd);
 				continue;
 			}
-			if (st->devnum < 0) {
+			if (st->devnum == MAXINT) {
 				struct stat stb;
 				if (fstat(fd, &stb) == 0 &&
-				    (S_IFMT&stb.st_mode)==S_IFBLK)
-					st->devnum = MINOR(stb.st_rdev);
+				    (S_IFMT&stb.st_mode)==S_IFBLK) {
+					if (MAJOR(stb.st_rdev) == MD_MAJOR)
+						st->devnum = MINOR(stb.st_rdev);
+					else
+						st->devnum = -1- (MINOR(stb.st_rdev)>>6);
+				}
 			}
 
-			for (mse = mdstat ; mse ; mse=mse->next)
-				if (mse->devnum == st->devnum) {
-					mse->devnum = -1; /* flag it as "used" */
-					break;
+			for (mse2 = mdstat ; mse2 ; mse2=mse2->next)
+				if (mse2->devnum == st->devnum) {
+					mse2->devnum = MAXINT; /* flag it as "used" */
+					mse = mse2;
 				}
 
 			if (st->utime == array.utime &&
@@ -241,6 +252,16 @@ int Monitor(mddev_dev_t devlist,
 				st->err = 0;
 				continue;
 			}
+			if (st->utime == 0 && /* new array */
+			    mse &&	/* is in /proc/mdstat */
+			    mse->pattern && strchr(mse->pattern, '_') /* degraded */
+				)
+				alert("DegradedArray", dev, NULL, mailaddr, alert_cmd);
+
+			if (st->utime == 0 && /* new array */
+			    st->expected_spares > 0 && 
+			    array.spare_disks < st->expected_spares) 
+				alert("SparesMissing", dev, NULL, mailaddr, alert_cmd);
 			if (mse &&
 			    st->percent == -1 && 
 			    mse->percent >= 0)
@@ -252,6 +273,11 @@ int Monitor(mddev_dev_t devlist,
 				alert(percentalerts[mse->percent/20],
 				      dev, NULL, mailaddr, alert_cmd);
 
+			if (mse &&
+			    mse->percent == -1 &&
+			    st->percent >= 0)
+				alert("RebuildFinished", dev, NULL, mailaddr, alert_cmd);
+
 			if (mse)
 				st->percent = mse->percent;
 					
@@ -271,19 +297,19 @@ int Monitor(mddev_dev_t devlist,
 					}
 				change = newstate ^ st->devstate[i];
 				if (st->utime && change && !st->err) {
-					if (i < array.raid_disks &&
+					if (i < (unsigned)array.raid_disks &&
 					    (((newstate&change)&(1<<MD_DISK_FAULTY)) ||
 					     ((st->devstate[i]&change)&(1<<MD_DISK_ACTIVE)) ||
 					     ((st->devstate[i]&change)&(1<<MD_DISK_SYNC)))
 						)
 						alert("Fail", dev, dv, mailaddr, alert_cmd);
-					else if (i>=array.raid_disks &&
+					else if (i >= (unsigned)array.raid_disks &&
 						 (disc.major || disc.minor) &&
 						 st->devid[i] == MKDEV(disc.major, disc.minor) &&
 						 ((newstate&change)&(1<<MD_DISK_FAULTY))
 						)
 						alert("FailSpare", dev, dv, mailaddr, alert_cmd);
-					else if (i < array.raid_disks &&
+					else if (i < (unsigned)array.raid_disks &&
 						 (((st->devstate[i]&change)&(1<<MD_DISK_FAULTY)) ||
 						  ((newstate&change)&(1<<MD_DISK_ACTIVE)) ||
 						  ((newstate&change)&(1<<MD_DISK_SYNC)))
@@ -306,23 +332,36 @@ int Monitor(mddev_dev_t devlist,
 		if (scan) {
 			struct mdstat_ent *mse;
 			for (mse=mdstat; mse; mse=mse->next) 
-				if (mse->devnum >= 0 &&
+				if (mse->devnum != MAXINT &&
 				    (strcmp(mse->level, "raid1")==0 ||
 				     strcmp(mse->level, "raid5")==0 ||
 				     strcmp(mse->level, "multipath")==0)
 					) {
 					struct state *st = malloc(sizeof *st);
+					mdu_array_info_t array;
+					int fd;
 					if (st == NULL)
 						continue;
 					st->devname = strdup(get_md_name(mse->devnum));
+					if ((fd = open(st->devname, O_RDONLY)) < 0 ||
+					    ioctl(fd, GET_ARRAY_INFO, &array)< 0) {
+						/* no such array */
+						if (fd >=0) close(fd);
+						free(st->devname);
+						free(st);
+						continue;
+					}
+					close(fd);
 					st->utime = 0;
 					st->next = statelist;
 					st->err = 1;
 					st->devnum = mse->devnum;
 					st->percent = -2;
 					st->spare_group = NULL;
+					st->expected_spares = -1;
 					statelist = st;
 					alert("NewArray", st->devname, NULL, mailaddr, alert_cmd);
+					new_found = 1;
 				}
 		}
 		/* If an array has active < raid && spare == 0 && spare_group != NULL
@@ -374,8 +413,13 @@ int Monitor(mddev_dev_t devlist,
 						close(fd2);
 					}
 			}
-					    
-		sleep(period);
+		if (!new_found) {
+			if (oneshot)
+				break;
+			else
+				mdstat_wait(period);
+		}
+		test = 0;
 	}
 	return 0;
 }
@@ -401,7 +445,10 @@ static void alert(char *event, char *dev, char *disc, char *mailaddr, char *cmd)
 			exit(2);
 		}
 	}
-	if (mailaddr && strncmp(event, "Fail", 4)==0) {
+	if (mailaddr && 
+	    (strncmp(event, "Fail", 4)==0 || 
+	     strncmp(event, "Test", 4)==0 ||
+	     strncmp(event, "Degrade", 7)==0)) {
 		FILE *mp = popen(Sendmail, "w");
 		if (mp) {
 			char hname[256];
