@@ -29,7 +29,9 @@
 
 #define	__USE_LARGEFILE64
 #include	<unistd.h>
+#ifndef __dietlibc__
 extern __off64_t lseek64 __P ((int __fd, __off64_t __offset, int __whence));
+#endif
 
 #include	<sys/types.h>
 #include	<sys/stat.h>
@@ -40,6 +42,12 @@ extern __off64_t lseek64 __P ((int __fd, __off64_t __offset, int __whence));
 #include	<stdio.h>
 #include	<errno.h>
 #include	<string.h>
+#ifdef __dietlibc__NONO
+int strncmp(const char *s1, const char *s2, size_t n) __THROW __pure__;
+char *strncpy(char *dest, const char *src, size_t n) __THROW;
+#include    <strings.h>
+#endif
+
 
 #include	<linux/kdev_t.h>
 /*#include	<linux/fs.h> */
@@ -47,9 +55,10 @@ extern __off64_t lseek64 __P ((int __fd, __off64_t __offset, int __whence));
 #include	<asm/types.h>
 #include	<sys/ioctl.h>
 #define	MD_MAJOR 9
+#define MdpMinorShift 6
 
 #ifndef BLKGETSIZE64
-#define BLKGETSIZE64 _IOR(0x12,114,sizeof(__u64)) /* return device size in bytes (u64 *arg) */
+#define BLKGETSIZE64 _IOR(0x12,114,size_t) /* return device size in bytes (u64 *arg) */
 #endif
 
 
@@ -65,12 +74,13 @@ enum mode {
 	MANAGE,
 	MISC,
 	MONITOR,
+	GROW,
 };
 
 extern char short_options[];
 extern struct option long_options[];
 extern char Version[], Usage[], Help[], OptionHelp[],
-	Help_create[], Help_build[], Help_assemble[],
+	Help_create[], Help_build[], Help_assemble[], Help_grow[],
 	Help_manage[], Help_misc[], Help_monitor[], Help_config[];
 
 /* structures read from config file */
@@ -83,21 +93,24 @@ extern char Version[], Usage[], Help[], OptionHelp[],
  * If multiple fields are present, the intersection of all matching
  * devices is considered
  */
+#define UnSet (0xfffe)
 typedef struct mddev_ident_s {
-	char *devname;
+	char	*devname;
 	
-	int uuid_set;
-	__u32 uuid[4];
+	int	uuid_set;
+	__u32	uuid[4];
 
-	int super_minor;	/* -1 if not set */
+	unsigned int super_minor;
 
-	char *devices;		/* comma separated list of device
+	char	*devices;	/* comma separated list of device
 				 * names with wild cards
 				 */
-	int level;		/* -10 if not set */
-	int raid_disks;		/* -1 if not set */
-	int spare_disks;	/* -1 if not set */
-	char *spare_group;
+	int	level;
+	unsigned int raid_disks;
+	unsigned int spare_disks;
+	int	autof;		/* 1 for normal, 2 for partitioned */
+	char	*spare_group;
+
 	struct mddev_ident_s *next;
 } *mddev_ident_t;
 
@@ -126,8 +139,9 @@ struct mdstat_ent {
 	struct mdstat_ent *next;
 };
 
-extern struct mdstat_ent *mdstat_read(void);
+extern struct mdstat_ent *mdstat_read(int);
 extern void free_mdstat(struct mdstat_ent *ms);
+extern void mdstat_wait(int seconds);
 
 #ifndef Sendmail
 #define Sendmail "/usr/lib/sendmail -t"
@@ -142,6 +156,7 @@ extern char *map_dev(int major, int minor);
 
 extern int Manage_ro(char *devname, int fd, int readonly);
 extern int Manage_runstop(char *devname, int fd, int runstop);
+extern int Manage_resize(char *devname, int fd, long long size, int raid_disks);
 extern int Manage_subdevs(char *devname, int fd,
 			  mddev_dev_t devlist);
 
@@ -156,7 +171,7 @@ extern int Assemble(char *mddev, int mdfd,
 
 extern int Build(char *mddev, int mdfd, int chunk, int level,
 		 int raiddisks,
-		 mddev_dev_t devlist);
+		 mddev_dev_t devlist, int assume_clean);
 
 
 extern int Create(char *mddev, int mdfd,
@@ -170,7 +185,7 @@ extern int Examine(mddev_dev_t devlist, int brief, int scan, int SparcAdjust);
 extern int Monitor(mddev_dev_t devlist,
 		   char *mailaddr, char *alert_cmd,
 		   int period, int daemonise, int scan, int oneshot,
-		   char *config);
+		   char *config, int test);
 
 extern int Kill(char *dev, int force);
 
@@ -181,18 +196,23 @@ extern int check_ext2(int fd, char *name);
 extern int check_reiser(int fd, char *name);
 extern int check_raid(int fd, char *name);
 
+extern int get_mdp_major(void);
+extern int is_standard(char *dev);
+
+
 extern mddev_ident_t conf_get_ident(char *conffile, char *dev);
 extern mddev_dev_t conf_get_devs(char *conffile);
 extern char *conf_get_mailaddr(char *conffile);
 extern char *conf_get_program(char *conffile);
 extern char *conf_line(FILE *file);
+extern char *conf_word(FILE *file, int allow_key);
 extern void free_line(char *line);
 extern int match_oneof(char *devices, char *devname);
 extern int load_super(int fd, mdp_super_t *super);
 extern void uuid_from_super(int uuid[4], mdp_super_t *super);
 extern int same_uuid(int a[4], int b[4]);
 extern int compare_super(mdp_super_t *first, mdp_super_t *second);
-extern int calc_sb_csum(mdp_super_t *super);
+extern unsigned long calc_sb_csum(mdp_super_t *super);
 extern int store_super(int fd, mdp_super_t *super);
 extern int enough(int level, int raid_disks, int avail_disks);
 extern int ask(char *mesg);
